package com.tzld.piaoquan.api.job;

import com.aliyun.odps.data.Record;
import com.google.common.collect.Lists;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ContentPlatformVideoAggMapper;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ContentPlatformVideoDataStatAggMapper;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ContentPlatformVideoDataStatMapper;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ContentPlatformVideoMapper;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ext.ContentPlatformPlanMapperExt;
import com.tzld.piaoquan.api.model.po.contentplatform.*;
import com.tzld.piaoquan.growth.common.model.bo.VideoDetail;
import com.tzld.piaoquan.growth.common.service.MessageAttachmentService;
import com.tzld.piaoquan.growth.common.utils.DateUtil;
import com.tzld.piaoquan.growth.common.utils.OdpsUtil;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Component
public class ContentPlatformVideoJob {

    @Autowired
    private ContentPlatformPlanMapperExt planMapperExt;

    @Autowired
    private ContentPlatformVideoMapper videoMapper;

    @Autowired
    private ContentPlatformVideoAggMapper videoAggMapper;

    @Autowired
    private ContentPlatformVideoDataStatMapper videoDataStatMapper;

    @Autowired
    private ContentPlatformVideoDataStatAggMapper videoDataStatAggMapper;

    @Autowired
    private MessageAttachmentService messageAttachmentService;

    @Value("${video.agg.days:3}")
    private Integer videoAggDays;

    @XxlJob("syncContentPlatformVideoJob")
    public ReturnT<String> syncContentPlatformVideoJob(String param) {
        String aggDt = DateUtil.getBeforeDayDateString("yyyyMMdd");
        if (StringUtils.hasText(param)) {
            aggDt = param;
        }
        List<String> dtList = DateUtil.getBeforeDays(aggDt, null, videoAggDays);
        // 轮询查询大数据获取最近14天视频
        for (String dt : dtList) {
            long videoCount = getVideoCount(dt);
            if (videoCount > 0) {
                continue;
            }
            String sql = String.format("SELECT * FROM loghubods.wecom_cooperation_video_candidate_pool WHERE dt=%s;", dt);
            List<Record> dataList = OdpsUtil.getOdpsData(sql);
            Long now = System.currentTimeMillis();
            if (CollectionUtils.isNotEmpty(dataList)) {
                List<ContentPlatformVideo> saveList = new ArrayList<>();
                for (Record record : dataList) {
                    ContentPlatformVideo item = new ContentPlatformVideo();
                    Long videoId = Long.parseLong((String) record.get(0));
                    String category = (String) record.get(1);
                    String title = (String) record.get(2);
                    String videoUrl = (String) record.get(3);
                    Double score = Double.parseDouble((String) record.get(4));
                    item.setDt(dt);
                    item.setVideoId(videoId);
                    item.setCategory(category);
                    item.setTitle(title);
                    item.setVideo(videoUrl);
                    item.setScore(score);
                    item.setCreateTimestamp(now);
                    saveList.add(item);
                }
                // 获取视频封面
                for (List<ContentPlatformVideo> partition : Lists.partition(saveList, 20)) {
                    List<Long> videoIds = partition.stream().map(ContentPlatformVideo::getVideoId).distinct().collect(Collectors.toList());
                    Map<Long, VideoDetail> coverMap = messageAttachmentService.getVideoDetail(new HashSet<>(videoIds));
                    for (ContentPlatformVideo item : partition) {
                        VideoDetail detail = coverMap.get(item.getVideoId());
                        if (Objects.nonNull(detail)) {
                            String cover = detail.getCover().substring(0, detail.getCover().indexOf("/watermark"));
                            item.setCover(cover);
                            if (detail.getAuditStatus() != 5 || detail.getRecommendStatus() != -6) {
                                item.setStatus(0);
                            }
                        }
                    }
                }
                // save
                if (CollectionUtils.isNotEmpty(saveList)) {
                    planMapperExt.batchInsertContentPlatformVideo(saveList);
                }
            }
        }
        // 更新封面
        List<ContentPlatformVideo> nullCoverVideoList = getVideoNullCover();
        if (CollectionUtils.isNotEmpty(nullCoverVideoList)) {
            for (List<ContentPlatformVideo> partition : Lists.partition(nullCoverVideoList, 20)) {
                List<Long> videoIds = partition.stream().map(ContentPlatformVideo::getVideoId).distinct().collect(Collectors.toList());
                Map<Long, VideoDetail> coverMap = messageAttachmentService.getVideoDetail(new HashSet<>(videoIds));
                for (ContentPlatformVideo item : partition) {
                    VideoDetail detail = coverMap.get(item.getVideoId());
                    if (Objects.isNull(detail)) {
                        continue;
                    }
                    String cover = detail.getCover().substring(0, detail.getCover().indexOf("/watermark"));
                    item.setCover(cover);
                    if (detail.getAuditStatus() != 5 || detail.getRecommendStatus() != -6) {
                        item.setStatus(0);
                    }
                    videoMapper.updateByPrimaryKeySelective(item);
                }
            }
        }
        long videoAggCount = getVideoAggCount(aggDt);
        if (videoAggCount > 0) {
            planMapperExt.deleteContentPlatformVideoAgg(aggDt);
        }
        // 聚合最近14天视频
        List<ContentPlatformVideoAgg> saveAggList = planMapperExt.getVideoAggList(dtList);
        if (CollectionUtils.isNotEmpty(saveAggList)) {
            Long now = System.currentTimeMillis();
            for (ContentPlatformVideoAgg item : saveAggList) {
                item.setDt(aggDt);
                item.setCreateTimestamp(now);
            }
            planMapperExt.batchInsertContentPlatformVideoAgg(saveAggList);
        }
        return ReturnT.SUCCESS;
    }

    private List<ContentPlatformVideo> getVideoNullCover() {
        ContentPlatformVideoExample example = new ContentPlatformVideoExample();
        example.createCriteria().andCoverIsNull();
        return videoMapper.selectByExample(example);
    }

    private long getVideoCount(String dt) {
        ContentPlatformVideoExample example = new ContentPlatformVideoExample();
        example.createCriteria().andDtEqualTo(dt);
        return videoMapper.countByExample(example);
    }

    private long getVideoAggCount(String dt) {
        ContentPlatformVideoAggExample example = new ContentPlatformVideoAggExample();
        example.createCriteria().andDtEqualTo(dt);
        return videoAggMapper.countByExample(example);
    }

    @XxlJob("checkContentPlatformVideoStatusJob")
    public ReturnT<String> checkContentPlatformVideoStatusJob(String param) {
        String dt = planMapperExt.getVideoMaxDt();
        if (StringUtils.hasText(param)) {
            dt = param;
        }
        List<ContentPlatformVideoAgg> videoList = getVideoListByDt(dt);
        Long now = System.currentTimeMillis();
        for (List<ContentPlatformVideoAgg> partition : Lists.partition(videoList, 20)) {
            List<Long> videoIds = partition.stream().map(ContentPlatformVideoAgg::getVideoId).collect(Collectors.toList());
            Map<Long, VideoDetail> videoDetailMap = messageAttachmentService.getVideoDetail(new HashSet<>(videoIds));
            for (ContentPlatformVideoAgg video : partition) {
                VideoDetail videoDetail = videoDetailMap.get(video.getVideoId());
                if (videoDetail == null) {
                    continue;
                }
                if (videoDetail.getAuditStatus() != 5 || videoDetail.getRecommendStatus() != -6) {
                    planMapperExt.updateVideoStatus(video.getVideoId(), now);
                    planMapperExt.updateVideoAggStatus(video.getVideoId(), now);
                }
            }
        }
        return ReturnT.SUCCESS;
    }

    private List<ContentPlatformVideoAgg> getVideoListByDt(String dt) {
        ContentPlatformVideoAggExample example = new ContentPlatformVideoAggExample();
        example.createCriteria().andDtEqualTo(dt).andStatusEqualTo(1);
        return videoAggMapper.selectByExample(example);
    }

    @XxlJob("syncContentPlatformVideoDatastatJob")
    public ReturnT<String> syncContentPlatformVideoDatastatJob(String param) {
        String aggDt = DateUtil.getBeforeDayDateString("yyyyMMdd");
        if (StringUtils.hasText(param)) {
            aggDt = param;
        }
        List<String> dtList = DateUtil.getBeforeDays(aggDt, null, 7);
        // 轮询查询大数据获取最近14天视频
        for (String dt : dtList) {
            long videoGroupScoreCount = getVideoGroupScoreCount(dt);
            if (videoGroupScoreCount > 0) {
                continue;
            }

            String sql = String.format("SELECT 行业, 渠道, vid, 首层访问人数, t0裂变人数, t0裂变率 " +
                    "FROM loghubods.wecom_cooperation_video_candidate_pool_hy_qd WHERE dt=%s;", dt);
            List<Record> dataList = OdpsUtil.getOdpsData(sql);
            Long now = System.currentTimeMillis();
            if (CollectionUtils.isNotEmpty(dataList)) {
                List<ContentPlatformVideoDataStat> saveList = new ArrayList<>();
                for (Record record : dataList) {
                    ContentPlatformVideoDataStat item = new ContentPlatformVideoDataStat();
                    String type =(String) record.get(0);
                    String channel =(String) record.get(1);
                    Long videoId = Long.parseLong((String) record.get(2));
                    Long firstLevel = Long.parseLong((String) record.get(3));
                    Long fission0 = Long.parseLong((String) record.get(4));
                    Double fissionRate = Double.parseDouble((String) record.get(5));
                    item.setDt(dt);
                    item.setVideoId(videoId);
                    item.setType(type);
                    item.setChannel(channel);
                    item.setFirstLevel(firstLevel);
                    item.setFission0(fission0);
                    item.setFissionRate(fissionRate);
                    item.setCreateTimestamp(now);
                    saveList.add(item);
                }
                // save
                if (CollectionUtils.isNotEmpty(saveList)) {
                    planMapperExt.batchInsertContentPlatformVideoDatastat(saveList);
                }
            }
        }
        // 聚合最近14天视频
        List<ContentPlatformVideoDataStatAgg> saveAggList = buildVideoDataStatAggList(aggDt, dtList);
        if (CollectionUtils.isNotEmpty(saveAggList)) {
            planMapperExt.deleteContentPlatformVideoDatastatAgg(aggDt);
            planMapperExt.batchInsertContentPlatformVideoDatastatAgg(saveAggList);
        }
        return ReturnT.SUCCESS;
    }

    private long getVideoGroupScoreCount(String dt) {
        ContentPlatformVideoDataStatExample example = new ContentPlatformVideoDataStatExample();
        example.createCriteria().andDtEqualTo(dt);
        return videoDataStatMapper.countByExample(example);
    }

    private List<ContentPlatformVideoDataStatAgg> buildVideoDataStatAggList(String aggDt, List<String> dtList) {
        Long now = System.currentTimeMillis();
        List<ContentPlatformVideoDataStatAgg> result = new ArrayList<>();
        for (ContentPlatformVideoDataStatAgg item : result) {
            item.setDt(aggDt);
            item.setCreateTimestamp(now);
        }
        return result;
    }

}



